

#include <t_services.h>
#include <s_services.h>
#include "kernel_id.h"
#include "rx_if.h"
#include <cdefBF533.h>


#define PEEPINGTEST             // Define PEEPING test to see the top 4 of FIFO after enabling SPORT RX

#ifdef PEEPINGTEST
#define SLEEPDURATION 1000
#else
#define SLEEPDURATION 10
#endif

struct dma_descripter{
    struct dma_descripter * next_descripter;
    unsigned int * start_address;
    unsigned short config;
    unsigned short x_count;
    unsigned short x_modify;
};

static struct dma_descripter rx_dma_dsc[2];

unsigned int rxif_buffer[2][RXIF_BUFSIZE];

/*
 *  RX_IF受信データの処理タスク。
 */
void rx_if_task(VP_INT exinf)
{

    vmsk_log(LOG_UPTO(LOG_INFO), LOG_UPTO(LOG_EMERG));
    syslog(LOG_NOTICE, "Sample program starts (exinf = %d).", (INT) exinf);

    syscall(serial_ctl_por(TASK_PORTID,
            (IOCTL_CRLF | IOCTL_FCSND | IOCTL_FCRCV)));



    syscall(ena_int(INTNO_SPORT0_RX));

    *pSPORT0_TCR1 =
            0 << 14 |   // TCKFE,   0:sample at down edge, 1:sample at up edge
            0 << 13 |   // LATFS,   0:early frame sync, 1:late frame sync
            0 << 12 |   // LTFS,    0:Active high TFS, 1:Active low TFS
            0 << 11 |   // DITFS,   0:data dependent TFS generation, 1:data independent TFS generation
            0 << 10 |   // TFSR,    0:TFS is not required every word, 1:TFS is required every word
            0 << 9  |   // ITFS,    0:external TFS, 1:internal TFS
            0 << 4  |   // TLSBIT,  0:MSB first transmission, 1:LSB first transmission
            0 << 2  |   // TDTYPE   0:normal, 1:reserved, 2:u-law, 3:a-law
            0 << 1  |   // ITCLK,   0:external clock generation, 1:internal clock generation
            0 << 0  ;   // TSPEN    0:Tx disable, 1:Tx enable

    *pSPORT0_TCR2 =
            0 << 10 |   // TRFST,   0:left streo ch first, 1:right stereo ch first
            0 << 9  |   // TSFESE,  0:normal frame sync, 1:LR frame clock
            0 << 8  |   // TXSE,    0:secondary ch disable, 1:secondary ch enable
            0 << 0  ;   // SLEN     0-1:not allowed,2-31:Serial word length - 1


    *pSPORT0_RCR1 =
            0 << 14 |   // RCKFE,   0:sample at down edge, 1:sample at up edge
            0 << 13 |   // LARFS,   0:early frame sync, 1:late frame sync
            0 << 12 |   // LRFS,    0:Active high RFS, 1:Active low RFS
            1 << 10 |   // RFSR,    0:RFS is not required every word, 1:RFS is required every word
            0 << 9  |   // IRFS,    0:external RFS, 1:internal RFS
            0 << 4  |   // RLSBIT,  0:MSB first transmission, 1:LSB first transmission
            0 << 2  |   // RDTYPE   0:zero fill, 1:sign extend, 2:u-law, 3:a-law
            0 << 1  |   // IRCLK,   0:external clock generation, 1:internal clock generation
            0 << 0  ;   // RSPEN    0:Rx disable, 1:Rx enable

    *pSPORT0_RCR2 =
            0 << 10 |   // RRFST,   0:left streo ch first, 1:right stereo ch first
            0 << 9  |   // RSFESE,  0:normal frame sync, 1:LR frame clock
            1 << 8  |   // RXSE,    0:secondary ch disable, 1:secondary ch enable
           29 << 0  ;   // SLEN     0-1:not allowed,2-31:Serial word length - 1
    ssync();



        // 受信DMAデスクリプタを作る
    rx_dma_dsc[0].next_descripter = &rx_dma_dsc[1];
    rx_dma_dsc[0].start_address = rxif_buffer[0];
    rx_dma_dsc[0].x_count = RXIF_BUFSIZE;
    rx_dma_dsc[0].x_modify = sizeof(rxif_buffer[0][0]);
    rx_dma_dsc[0].config =
            FLOW_LARGE      |   // FLOW,    0:Stop, 1:Auto buffer, 4:Desc array, 6:Desc List small, 7:Desc, List, Large
            NDSIZE_7        |   // NDSIZE,  the # of element of the next descripter to fetch
            1 << DI_EN_P    |   // DI_EN,   0:No interrupt at the end, 1:Interrupt at the end
            0 << DI_SEL_P   |   // DI_SEL,  0:Interrupt at the end of outer loop, 1:Interrupt at the end of inter loop
            0 << RESTART_P  |   // RESTART, 0:Keep DMA FIFO before start, 1:Purge DMA FIFO before start
            0 << DMA2D_P    |   // DMA2D,   0:Linear DMA, 1:2D DMA
            WDSIZE_32       |   // WDSIZE,  0:8bit, 1:16bit, 2:32bit,3:reserved
            1 << WNR_P      |   // WNR,     0:Read from memory, 1:Write to Memory
            1 << 0          ;   // DMA_EN,  0:Disable DMA, 1:Enable DMA

    rx_dma_dsc[1].next_descripter = &rx_dma_dsc[0];
    rx_dma_dsc[1].start_address = rxif_buffer[1];
    rx_dma_dsc[1].x_count = RXIF_BUFSIZE;
    rx_dma_dsc[1].x_modify = sizeof(rxif_buffer[1][0]);
    rx_dma_dsc[1].config =
            FLOW_LARGE      |   // FLOW,    0:Stop, 1:Auto buffer, 4:Desc array, 6:Desc List small, 7:Desc, List, Large
            NDSIZE_7        |   // NDSIZE,  the # of element of the next descripter to fetch
            1 << DI_EN_P    |   // DI_EN,   0:No interrupt at the end, 1:Interrupt at the end
            0 << DI_SEL_P   |   // DI_SEL,  0:Interrupt at the end of outer loop, 1:Interrupt at the end of inter loop
            0 << RESTART_P  |   // RESTART, 0:Keep DMA FIFO before start, 1:Purge DMA FIFO before start
            0 << DMA2D_P    |   // DMA2D,   0:Linear DMA, 1:2D DMA
            WDSIZE_32       |   // WDSIZE,  0:8bit, 1:16bit, 2:32bit,3:reserved
            1 << WNR_P      |   // WNR,     0:Read from memory, 1:Write to Memory
            1 << 0          ;   // DMA_EN,  0:Disable DMA, 1:Enable DMA

        // DMAコントローラの初期状態設定
        // ここではDMAをイネーブルにしない。また、バッファクリアする
    *pDMA1_CONFIG =
            FLOW_LARGE      |   // FLOW,    0:Stop, 1:Auto buffer, 4:Desc array, 6:Desc List small, 7:Desc, List, Large
            NDSIZE_7        |   // NDSIZE,  the # of element of the next descripter to fetch
            1 << DI_EN_P    |   // DI_EN,   0:No interrupt at the end, 1:Interrupt at the end
            0 << DI_SEL_P   |   // DI_SEL,  0:Interrupt at the end of outer loop, 1:Interrupt at the end of inter loop
            1 << RESTART_P  |   // RESTART, 0:Keep DMA FIFO before start, 1:Purge DMA FIFO before start
            0 << DMA2D_P    |   // DMA2D,   0:Linear DMA, 1:2D DMA
            WDSIZE_32       |   // WDSIZE,  0:8bit, 1:16bit, 2:32bit,3:reserved
            1 << WNR_P      |   // WNR,     0:Read from memory, 1:Write to Memory
            0 << 0          ;   // DMA_EN,  0:Disable DMA, 1:Enable DMA
    *pDMA1_NEXT_DESC_PTR = &rx_dma_dsc[0];


#ifndef PEEPINGTEST
    int count = 0, event = 0;
#endif


        // RX受信を開始する
    *pDMA1_CONFIG |= DMAEN;

    *pSPORT0_RCR1 |= RSPEN;     // RX SPORT Enable
    ssync();

    int count = 0;

    /*
     *  メインループ
     */
    do {
        long int rx[4];

            // SPORT0受信DMAがバッファを埋めるのを待つ。
        syscall(wai_sem(SEM_SPORT0_RX));

        count++;
        if ( count > 31500/(RXIF_BUFSIZE/4))
        {
            count = 0;
                // FIFOにデータがたまったはずである。
                // FIFOデータを読みだして下位2bitのみ表示する
            rx[0] = rxif_buffer[0][0];    // 下位2bitのみ抽出
            rx[1] = rxif_buffer[0][1];    // 下位2bitのみ抽出
            rx[2] = rxif_buffer[0][2];    // 下位2bitのみ抽出
            rx[3] = rxif_buffer[0][3];    // 下位2bitのみ抽出

            syslog( LOG_NOTICE, "RX word : %08x,%08x,%08x,%08x", rx[0], rx[1], rx[2], rx[3] );

        }





    } while (1);

    syslog(LOG_NOTICE, "Sample program ends.");
    kernel_exit();
}


void sport0_rx_int_handler(void)
{
        // DMA割り込みをクリアする。
    *pDMA1_IRQ_STATUS = DMA_DONE;

        // ペリフェラルへの書き込みを待つ。
    ssync();

        // タスクにSPORT0受信DMAのバッファが埋まったと知らせる。
    isig_sem(SEM_SPORT0_RX);


}

void sport0_tx_int_handler(void)
{
    *pDMA2_IRQ_STATUS = DMA_DONE;
    ssync();


}


void init_peripherals(VP_INT p)
{
    *pSPORT0_TCR1 = 0;
    *pSPORT0_RCR1 = 0;
    *pDMA1_CONFIG = 0;
    *pDMA2_CONFIG = 0;
    ssync();

    return;
}
